@@ -4,16 +4,16 @@
  * This file defines constants used internally by the calma
  * module, but not exported to the rest of the world.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  *
  * rcsid $Header: /usr/cvsroot/magic-8.0/calma/calmaInt.h,v 1.2 2010/06/24 12:37:15 tim Exp $
@@ -99,6 +99,7 @@
 #define	CALMA_NUMRECORDTYPES	60	/* Number of above types */
 
 /* Property types defined for magic */
+#define CALMA_PROP_USENAME_STD	61	/* To record non-default cell use ids */
 #define CALMA_PROP_USENAME	98	/* To record non-default cell use ids */
 #define CALMA_PROP_ARRAY_LIMITS	99	/* To record non-default array limits */
 
@@ -130,6 +131,9 @@ typedef struct
 /* Length of record header */
 #define	CALMAHEADERLENGTH	4
 
+/* Label types */
+typedef enum { LABEL_TYPE_NONE, LABEL_TYPE_TEXT, LABEL_TYPE_PORT, LABEL_TYPE_CELLID } labelType;
+
 /* ------------------------- Input macros ----------------------------- */
 
 /* Globals for Calma reading */
@@ -154,7 +158,7 @@ extern int  calmaLArtype;
 # define ntohs(x)        (x)
 # define htonl(x)        (x)
 # define htons(x)        (x)
-# endif 
+# endif
 #endif
 
 typedef union { char uc[2]; unsigned short us; } TwoByteInt;
