@@ -625,6 +625,12 @@ CmdErase(w, cmd)
 
     if (!ToolGetEditBox(&editRect)) return;
 
+    if (EditCellUse == NULL)
+    {
+        TxError("No cell def being edited!\n");
+	return;
+    }
+
     /*
      * Erase with no arguments is the same as erasing
      * everything underneath the box tool (ie, painting space)
@@ -890,6 +896,7 @@ cmdExpandFunc(use, windowMask)
 #define UNIQALL		0
 #define UNIQTAGGED	1
 #define UNIQNOPORTS	2
+#define UNIQNOTOPPORTS	3
 
 void
 CmdExtract(w, cmd)
@@ -1006,7 +1013,10 @@ CmdExtract(w, cmd)
 
 	if (argc == 1)
 	{
-	    ExtIncremental(selectedUse);
+	    if (!strcmp(selectedUse->cu_def->cd_name, UNNAMED))
+		TxError("Please name the cell before extracting.\n");
+	    else
+		ExtIncremental(selectedUse);
 	    return;
 	}
     }
@@ -1037,7 +1047,10 @@ CmdExtract(w, cmd)
 	    break;
 
 	case EXTALL:
-	    ExtAll(selectedUse);
+	    if (!strcmp(selectedUse->cu_def->cd_name, UNNAMED))
+		TxError("Please name the cell before extracting.\n");
+	    else
+		ExtAll(selectedUse);
 	    return;
 
 	case EXTCELL:
@@ -1059,7 +1072,10 @@ CmdExtract(w, cmd)
 		TxError("No cell selected\n");
 		return;
 	    }
-	    ExtParents(selectedUse);
+	    if (!strcmp(selectedUse->cu_def->cd_name, UNNAMED))
+		TxError("Please name the cell before extracting.\n");
+	    else
+		ExtParents(selectedUse);
 	    return;
 
 	case EXTSHOWPARENTS:
