@@ -91,17 +91,18 @@ DBIsSubcircuit(cellDef)
  */
 
 Label *
-DBPutLabel(cellDef, rect, align, text, type, flags)
+DBPutLabel(cellDef, rect, align, text, type, flags, port)
     CellDef *cellDef;
     Rect *rect;
     int align;
     char *text;
     TileType type;
-    int flags;
+    unsigned short flags;
+    unsigned int port;
 {
     /* Draw text in a standard X11 font */
     return DBPutFontLabel(cellDef, rect, -1, 0, 0, &GeoOrigin,
-		align, text, type, flags);
+		align, text, type, flags, port);
 }
 
 /*
@@ -128,7 +129,7 @@ DBPutLabel(cellDef, rect, align, text, type, flags)
  * ----------------------------------------------------------------------------
  */
 Label *
-DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
+DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags, port)
     CellDef *cellDef;	/* Cell in which label is placed */
     Rect *rect;		/* Location of label; see above for description */
     int font;		/* A vector outline font to use, or -1 for X11 font */
@@ -141,7 +142,8 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
 			 */
     char *text;		/* Pointer to actual text of label */
     TileType type;	/* Type of tile to be labelled */
-    int flags;		/* Label flags */
+    unsigned short flags; /* Label flags */
+    unsigned int port;	/* Port index (if label is a port, per the flags) */
 {
     Label *lab;
     int len, x1, x2, y1, y2, tmp, labx, laby;
@@ -208,6 +210,7 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
     }
     lab->lab_type = type;
     lab->lab_flags = flags;
+    lab->lab_port = port;
     lab->lab_rect = *rect;
     lab->lab_next = NULL;
     if (cellDef->cd_labels == NULL)
@@ -228,7 +231,7 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
 /*
  * ----------------------------------------------------------------------------
  *
- * DBEraseLabel --
+ * DBEraseGlobLabel --
  *
  * Delete labels attached to tiles of the indicated types that
  * are in the given area (as determined by the macro GEO_LABEL_IN_AREA).
@@ -250,13 +253,13 @@ DBPutFontLabel(cellDef, rect, font, size, rot, offset, align, text, type, flags)
  *	still enough material to keep them around.  The rect pointed to
  *	by areaReturn is filled with the area affected by removing the
  *	label, for purposes of redrawing the necessary portions of the
-*	screen.
+ *	screen.
  *
  * ----------------------------------------------------------------------------
  */
 
 bool
-DBEraseLabel(cellDef, area, mask, areaReturn)
+DBEraseGlobLabel(cellDef, area, mask, areaReturn, globmatch)
     CellDef *cellDef;		/* Cell being modified */
     Rect *area;			/* Area from which labels are to be erased.
 				 * This may be a point; any labels touching
@@ -266,6 +269,9 @@ DBEraseLabel(cellDef, area, mask, areaReturn)
 				 * be erased.
 				 */
     Rect *areaReturn;		/* Expand this with label bounding box */
+    char *globmatch;		/* If non-NULL, do glob-style matching of
+				 * any label against this string.
+				 */
 {
     Label *lab, *labPrev;
     bool erasedAny = FALSE;
@@ -289,6 +295,10 @@ DBEraseLabel(cellDef, area, mask, areaReturn)
 		if (DBConnectsTo(newType, lab->lab_type)) goto nextLab;
 	    }
 	}
+	if (globmatch != NULL)
+	{
+	    if (!Match(globmatch, lab->lab_text)) goto nextLab;
+	}
 
 	DBWLabelChanged(cellDef, lab, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
@@ -319,6 +329,39 @@ DBEraseLabel(cellDef, area, mask, areaReturn)
 				&& (r1)->r_xtop == (r2)->r_xtop \
 				&& (r1)->r_ytop == (r2)->r_ytop)
 
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * DBEraseLabel ---
+ *
+ *	Wrapper around DBEraseGlobLabel() with globmatch set to NULL so
+ *	that labels are erased verbatim rather that being matched against
+ *	a glob-style string.
+ *
+ * Results:
+ *	Passes back the result of DBEraseGlobLabel()
+ *
+ * Side effects:
+ *	See DBEraseGlobLabel()
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+DBEraseLabel(cellDef, area, mask, areaReturn)
+    CellDef *cellDef;		/* Cell being modified */
+    Rect *area;			/* Area from which labels are to be erased.
+				 * This may be a point; any labels touching
+				 * or overlapping it are erased.
+				 */
+    TileTypeBitMask *mask;	/* Mask of types from which labels are to
+				 * be erased.
+				 */
+    Rect *areaReturn;		/* Expand this with label bounding box */
+{
+    return DBEraseGlobLabel(cellDef, area, mask, areaReturn, NULL);
+}
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -426,9 +469,9 @@ DBEraseLabelsByContent(def, rect, type, text)
 /*
  * ----------------------------------------------------------------------------
  *
- * DBEraseLabelsByFunction --
+ * DBRemoveLabel --
  *
- * Erase any labels found on the label list for which the function returns
+ * Erase a labels by reference.
  * TRUE.
  *
  * Results:
@@ -443,19 +486,9 @@ DBEraseLabelsByContent(def, rect, type, text)
  */
 
 void
-DBEraseLabelsByFunction(def, func)
+DBRemoveLabel(def, refLab)
     CellDef *def;		/* Where to look for label to delete. */
-    bool (*func)();		/* Function to call for each label.  If it
-				 * returns TRUE, we delete the label.
-				 *
-				 * Function should be of the form:
-				 *
-				 *	bool func(lab)
-				 *	    Label *lab;
-				 *	{
-				 *	    return XXX;
-				 *	}
-				 */
+    Label *refLab;
 {
     Label *lab, *labPrev;
 
@@ -464,7 +497,7 @@ DBEraseLabelsByFunction(def, func)
 	    labPrev = lab, lab = lab->lab_next)
     {
 	nextCheck:
-	if (!(*func)(lab)) continue;
+	if (lab != refLab) continue;
 	DBUndoEraseLabel(def, lab);
 	DBWLabelChanged(def, lab, DBW_ALLWINDOWS);
 	if (labPrev == NULL)
