@@ -3,21 +3,21 @@
  *
  * Interface to the undo package for the database.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
 #ifndef lint
-static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBundo.c,v 1.4 2010/06/24 12:37:15 tim Exp $";
+static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/database/DBundo.c,v 1.5 2010/09/24 19:53:19 tim Exp $";
 #endif  /* not lint */
 
 #include <stdio.h>
@@ -107,7 +107,7 @@ bool dbUndoUndid;
 
 extern void dbUndoEdit();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -204,7 +204,7 @@ DBUndoReset(celldef)
     }
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -229,7 +229,7 @@ dbUndoInit()
     dbUndoAreaChanged.r_xbot = dbUndoAreaChanged.r_xtop = 0;
     dbUndoAreaChanged.r_ybot = dbUndoAreaChanged.r_ytop = 0;
 }
-
+
 /*
  * ============================================================================
  *
@@ -259,6 +259,7 @@ dbUndoSplitForw(us)
     splitUE *us;
 {
     /* Create internal fracture */
+    if (dbUndoLastCell == NULL) return;
     DBSplitTile(dbUndoLastCell->cd_planes[us->sue_plane], &us->sue_point,
 		us->sue_splitx);
 }
@@ -268,6 +269,7 @@ dbUndoSplitBack(us)
     splitUE *us;
 {
     Rect srect;
+    if (dbUndoLastCell == NULL) return;
 
     srect.r_ll = us->sue_point;
     srect.r_ur.p_x = us->sue_point.p_x + 1;
@@ -305,6 +307,7 @@ dbUndoPaintForw(up)
     paintUE *up;
 {
     TileType loctype, dinfo;
+    if (dbUndoLastCell == NULL) return;
 
     if (up->pue_oldtype & TT_DIAGONAL)
     {
@@ -358,6 +361,7 @@ dbUndoPaintBack(up)
     paintUE *up;
 {
     TileType loctype, dinfo;
+    if (dbUndoLastCell == NULL) return;
 
     if (up->pue_newtype & TT_DIAGONAL)
     {
@@ -409,7 +413,7 @@ endPaintBack:
     (void) GeoInclude(&up->pue_rect, &dbUndoAreaChanged);
     (void) DRCCheckThis(dbUndoLastCell, TT_CHECKPAINT, &up->pue_rect);
 }
-
+
 /*
  * ============================================================================
  *
@@ -430,6 +434,7 @@ typedef  Label labelUE;
 #define  lue_offset lab_offset
 #define  lue_flags  lab_flags
 #define  lue_text   lab_text
+#define  lue_port   lab_port
 
     /*
      * labelSize(n) is the size of a labelUE large enough to hold
@@ -481,7 +486,7 @@ DBUndoPutLabel(cellDef, lab)
     lup->lue_offset = lab->lab_offset;
     strcpy(lup->lue_text, lab->lab_text);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -524,7 +529,7 @@ DBUndoEraseLabel(cellDef, lab)
     lup->lue_offset = lab->lab_offset;
     strcpy(lup->lue_text, lab->lab_text);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -548,10 +553,11 @@ dbUndoLabelForw(up)
 {
     Label *lab;
 
+    if (dbUndoLastCell == NULL) return;
     lab = DBPutFontLabel(dbUndoLastCell, &up->lue_rect,
 	up->lue_font, up->lue_size, up->lue_rotate,
 	&up->lue_offset, up->lue_just, up->lue_text,
-	up->lue_type, up->lue_flags);
+	up->lue_type, up->lue_flags, up->lue_port);
     DBWLabelChanged(dbUndoLastCell, lab, DBW_ALLWINDOWS);
 
     /*
@@ -569,6 +575,7 @@ void
 dbUndoLabelBack(up)
     labelUE *up;
 {
+    if (dbUndoLastCell == NULL) return;
     (void) DBEraseLabelsByContent(dbUndoLastCell, &up->lue_rect,
 		up->lue_type, up->lue_text);
 
@@ -579,7 +586,7 @@ dbUndoLabelBack(up)
     if (up->lue_type == TT_SPACE)
 	dbUndoUndid = TRUE;
 }
-
+
 /*
  * ============================================================================
  *
@@ -664,7 +671,7 @@ DBUndoCellUse(use, action)
     up->cue_flags = use->cu_flags;
     strcpy(up->cue_id, use->cu_id);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -697,7 +704,7 @@ dbUndoCellForw(up)
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_extended = up->cue_extended;
-	    use->cu_flags = up->cue_flags; 
+	    use->cu_flags = up->cue_flags;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
 	    (void) DBLinkCell(use, up->cue_parent);
 	    DBPlaceCell(use, up->cue_parent);
@@ -741,7 +748,7 @@ dbUndoCellForw(up)
 	    freeMagic(use->cu_id);
 	    use->cu_id = (char *) NULL;
 	    break;
-	
+
 	case UNDO_CELL_LOCKDOWN:
 	    use = findUse(up, TRUE);
 	    use->cu_flags = up->cue_flags;
@@ -766,7 +773,7 @@ dbUndoCellBack(up)
 	    use->cu_expandMask = up->cue_expandMask;
 	    use->cu_bbox = up->cue_bbox;
 	    use->cu_extended = up->cue_extended;
-	    use->cu_flags = up->cue_flags; 
+	    use->cu_flags = up->cue_flags;
 	    use->cu_id = StrDup((char **) NULL, up->cue_id);
 	    SigDisableInterrupts ();
 	    (void) DBLinkCell(use, up->cue_parent);
@@ -821,7 +828,7 @@ dbUndoCellBack(up)
 	    break;
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -842,7 +849,7 @@ dbUndoCellBack(up)
  *
  * ----------------------------------------------------------------------------
  */
- 
+
 CellUse *
 findUse(up, matchName)
     cellUE *up;
@@ -868,7 +875,7 @@ findUse(up, matchName)
     ASSERT(FALSE, "findUse: use == NULL");
     return (CellUse *) NULL;
 }
-
+
 /*
  * ============================================================================
  *
@@ -935,7 +942,7 @@ dbUndoEdit(new)
     strcpy(up->eue_name, new->cd_name);
     dbUndoLastCell = new;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -962,7 +969,7 @@ dbUndoOpenCell(eup)
     ASSERT(newDef != (CellDef *) NULL, "dbUndoOpenCell");
     dbUndoLastCell = newDef;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
