@@ -638,8 +638,8 @@ runexttosim:
     (void) sprintf( esCapFormat, " GND %%.%dlf\n", esCapAccuracy);
     EFVisitNodes(simnodeVisit, (ClientData) NULL);
 
-    EFFlatDone();
-    EFDone();
+    EFFlatDone(NULL);
+    EFDone(NULL);
 
     if (esSimF) fclose(esSimF);
     if (esLabF) fclose(esLabF);
@@ -766,8 +766,8 @@ main(argc, argv)
     (void) sprintf( esCapFormat, " GND %%.%dlf\n", esCapAccuracy);
     EFVisitNodes(simnodeVisit, (ClientData) NULL);
 
-    EFFlatDone();
-    EFDone();
+    EFFlatDone(NULL);
+    EFDone(NULL);
 
     if (esSimF) fclose(esSimF);
     if (esLabF) fclose(esLabF);
@@ -1045,7 +1045,8 @@ simdevVisit(dev, hc, scale, trans)
 	case DEV_DIODE:
 	case DEV_PDIODE:
 	case DEV_NDIODE:
-	    fprintf(esSimF, "D");
+	    is_subckt = TRUE;
+	    fprintf(esSimF, "x");	/* no class for diodes, use subckt */
 	    break;
 	case DEV_RES:
 	    fprintf(esSimF, "r");
