@@ -1884,7 +1884,7 @@ esHierVisit(hc, cdata)
     if ((def != topdef) && (hc->hc_use->use_def->def_flags & DEF_NODEVICES) &&
 		(!doStub))
     {
-	EFFlatDone();
+	EFFlatDone(esFreeNodeClient);
 	return 0;
     }
     else if (doStub)
@@ -1979,13 +1979,15 @@ esHierVisit(hc, cdata)
     /* Reset the subcircuit hash table, if using HSPICE format */
     if (esFormat == HSPICE)
     {
+	HashKill(&subcktNameTable);
         HashInit(&subcktNameTable, 32, HT_STRINGKEYS);
 #ifndef UNSORTED_SUBCKT
+        DQFree(&subcktNameQueue);
         DQInit(&subcktNameQueue, 64);
 #endif
     }
 
-    EFFlatDone();
+    EFFlatDone(esFreeNodeClient);
     return 0;
 }
 
