@@ -4,16 +4,16 @@
  * General command-line argument processing and overall initialization
  * for the .ext file flattener.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -43,7 +43,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
     /* Command-line flags */
 EFCapValue EFCapThreshold = 2;  /* -c/-C: (fF) smallest interesting C */
 int EFResistThreshold = 10;	/* -r/-R: (Ohms) smallest interesting R */
-int EFTrimFlags = 0;		/* -t: output of nodename trailing #!'s */
+int EFOutputFlags = 0;		/* -t: output of nodename trailing #!'s */
 char *EFSearchPath = NULL;	/* -p: Search path for .ext files */
 char *EFArgTech = NULL;		/* -T: Tech specified on command line */
 
@@ -81,7 +81,7 @@ bool efScaleChanged = FALSE;	/* TRUE if .ext scales mismatch */
 extern int  efLoadPathFunc();
 extern void efLoadSearchPath();
 #endif
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -160,6 +160,15 @@ EFArgs(argc, argv, err_result, argsProc, cdata)
     HierName *hierName;
     FILE *f;
 
+    char usage_text[] =
+	"Standard arguments: [-R] [-C] [-r rthresh] [-c cthresh] [-v]\n"
+		"[-p searchpath] [-s sym=value] [-S symfile] [-t trimchars]\n"
+#ifdef MAGIC_WRAPPER
+		"[rootfile]\n";
+#else
+		"[-T techname] rootfile\n";
+#endif
+
     if (err_result != NULL) *err_result = FALSE;
 
     /* Hash table of nodes we're going to watch if -N given */
@@ -207,13 +216,15 @@ EFArgs(argc, argv, err_result, argsProc, cdata)
 	    case 't':
 		if ((cp = ArgStr(&argc, &argv, "trim characters")) == NULL)
 		    goto usage;
-		if (strchr(cp, '!')) EFTrimFlags |= EF_TRIMGLOB;
-		if (strchr(cp, '#')) EFTrimFlags |= EF_TRIMLOCAL;
-		if (strchr(cp, ',')) EFTrimFlags |= EF_CONVERTCOMMAS;
-		if (strchr(cp, '=')) EFTrimFlags |= EF_CONVERTEQUAL;
+		if (strchr(cp, '!')) EFOutputFlags |= EF_TRIMGLOB;
+		if (strchr(cp, '#')) EFOutputFlags |= EF_TRIMLOCAL;
+		if (strchr(cp, ',')) EFOutputFlags |= EF_CONVERTCOMMA;
+		if (strchr(cp, '=')) EFOutputFlags |= EF_CONVERTEQUAL;
+		if (strchr(cp, '[')) EFOutputFlags |= EF_CONVERTBRACKETS;
+		if (strchr(cp, ']')) EFOutputFlags |= EF_CONVERTBRACKETS;
 		break;
 	    case 'C':
-		EFCapThreshold = INFINITE_THRESHOLD;
+		EFCapThreshold = (EFCapValue)INFINITE_THRESHOLD_F;
 		break;
 	    case 'R':
 		EFResistThreshold = INFINITE_THRESHOLD;
@@ -268,6 +279,11 @@ EFArgs(argc, argv, err_result, argsProc, cdata)
 	    case 'z':
 		efHNStats = TRUE;
 		break;
+	    case 'h':
+		if (argsProc != NULL) (*argsProc)(&argc, &argv, cdata);
+		TxPrintf(usage_text);
+		if (err_result != NULL) *err_result = TRUE;
+		return NULL;
 
 	    /*** Try a caller-supplied argument processing function ***/
 	    default:
@@ -310,26 +326,22 @@ EFArgs(argc, argv, err_result, argsProc, cdata)
 	realIn[cp - inname] = '\0';
 	inname = realIn;
     }
-
     return inname;
 
 usage:
-    TxError("Standard arguments: [-R] [-C] [-r rthresh] [-c cthresh] [-v]\n"
-		"[-p searchpath] [-s sym=value] [-S symfile] [-t trimchars]\n"
+    TxError(usage_text);
 
 #ifdef MAGIC_WRAPPER
-		"[rootfile]\n");
     if (err_result != NULL) *err_result = TRUE;
     return NULL;
 #else
-		"[-T techname] rootfile\n");
     exit (1);
     /*NOTREACHED*/
 #endif
 }
 
 #ifndef MAGIC_WRAPPER
-
+
 /*
  * ----------------------------------------------------------------------------
  *
