@@ -1122,13 +1122,7 @@ EFGetPortMax(def, imp_max)
                 snode != &def->def_firstn;
                 snode = (EFNode *) snode->efnode_next)
     {
-        if (imp_max && (snode->efnode_flags & EF_SUBS_PORT))
-	{
-	    nodeName = snode->efnode_name;
-	    portorder = nodeName->efnn_port;
-	    if (portorder > (*imp_max)) (*imp_max) = portorder;
-	}
-        else if (snode->efnode_flags & EF_PORT)
+        if (snode->efnode_flags & EF_PORT)
 	{
 	    for (nodeName = snode->efnode_name; nodeName != NULL; nodeName =
                         nodeName->efnn_next)
@@ -1137,7 +1131,15 @@ EFGetPortMax(def, imp_max)
 		if (portorder > portmax) portmax = portorder;
 	    }
 	}
+        else if (imp_max && (snode->efnode_flags & EF_SUBS_PORT))
+	{
+	    nodeName = snode->efnode_name;
+	    portorder = nodeName->efnn_port;
+	    if (portorder > (*imp_max)) (*imp_max) = portorder;
+	}
     }
+    if (imp_max)
+	if (portmax > (*imp_max)) (*imp_max) = portmax;
     return portmax;
 }
 
@@ -1169,6 +1171,7 @@ efBuildDevNode(def, name, isSubsNode)
 {
     HashEntry *he;
     EFNodeName *nn;
+    bool isNewNode = FALSE;
 
     he = HashFind(&def->def_nodes, name);
     nn = (EFNodeName *) HashGetValue(he);
@@ -1181,17 +1184,20 @@ efBuildDevNode(def, name, isSubsNode)
 		(char *) NULL, (char **) NULL, 0);
 
 	nn = (EFNodeName *) HashGetValue(he);
-	if (isSubsNode)
+	isNewNode = TRUE;
+    }
+    if (isSubsNode)
+    {
+	if (!EFHNIsGlob(nn->efnn_hier))
 	{
-	    if (!EFHNIsGlob(nn->efnn_hier))
-	    {
-		/* This node is declared to be an implicit port */
-		nn->efnn_node->efnode_flags |= EF_SUBS_PORT;
-		nn->efnn_port = -1;
-		def->def_flags |= DEF_SUBSNODES;
-	    }
-	    nn->efnn_node->efnode_flags |= (EF_DEVTERM | EF_SUBS_NODE);
+	    /* This node is declared to be an implicit port */
+	    nn->efnn_node->efnode_flags |= EF_SUBS_PORT;
+	    nn->efnn_port = -1;
+	    def->def_flags |= DEF_SUBSNODES;
 	}
+	nn->efnn_node->efnode_flags |= EF_SUBS_NODE;
+	if (isNewNode)
+	    nn->efnn_node->efnode_flags |= EF_DEVTERM;
     }
     return nn->efnn_node;
 }
@@ -1982,18 +1988,30 @@ efFreeNodeTable(table)
  * Free the circular list of nodes of which 'head' is the head.
  * Don't free 'head' itself, since it's statically allocated.
  *
+ * If the client (e.g., ext2spice, ext2sim, ...) allocates memory for a
+ * node, then that client needs to provide a function "func" of the form:
+ *
+ *              int func(client) 
+ *		    ClientData client;
+ *		{
+ *		}
+ *
+ * The return value is unused but should be zero for consistency.
+ *
  * Results:
  *	None.
  *
  * Side effects:
- *	Frees memory.
+ *	Frees memory.  Calls func() to free additional memory allocated
+ *	in the node structure by a client.
  *
  * ----------------------------------------------------------------------------
  */
 
 void
-efFreeNodeList(head)
+efFreeNodeList(head, func)
     EFNode *head;
+    int (*func)();
 {
     EFNode *node;
     EFAttr *ap;
@@ -2004,6 +2022,12 @@ efFreeNodeList(head)
     {
 	for (ap = node->efnode_attrs; ap; ap = ap->efa_next)
 	    freeMagic((char *) ap);
+	if (node->efnode_client != (ClientData)NULL)
+	{
+	    if (func != NULL)
+		(*func)(node->efnode_client);
+	    freeMagic((char *)node->efnode_client);
+	}
 	freeMagic((char *) node);
     }
 }
