@@ -130,7 +130,11 @@ EFFlatBuild(name, flags)
     if (flags & EF_FLATNODES)
     {
 	if (flags & EF_NOFLATSUBCKT)
+	{
+	    /* The top cell must always have the DEF_SUBCIRCUIT flag cleared */
+	    efFlatRootDef->def_flags &= ~DEF_SUBCIRCUIT;
 	    efFlatNodesStdCell(&efFlatContext);
+	}
 	else
 	{
 	    int flags = FLATNODE_DOWARN;    /* No FLATNODE_STDCELL flag */
@@ -236,7 +240,7 @@ EFFlatBuildOneLevel(def, flags)
  * EFFlatDone --
  *
  * Cleanup by removing all memory used by the flattened circuit
- * representation.
+ * representation.  For use of func(), see efFreeNodeList() in EFbuild.c.
  *
  * Results:
  *	None.
@@ -248,7 +252,8 @@ EFFlatBuildOneLevel(def, flags)
  */
 
 void
-EFFlatDone()
+EFFlatDone(func)
+    int (*func)();
 {
 #ifdef	MALLOCTRACE
     /* Hash table statistics */
@@ -258,9 +263,10 @@ EFFlatDone()
 
     /* Free temporary storage */
     efFreeNodeTable(&efNodeHashTable);
-    efFreeNodeList(&efNodeList);
+    efFreeNodeList(&efNodeList, func);
     HashFreeKill(&efCapHashTable);
     HashKill(&efNodeHashTable);
+    HashKill(&efDistHashTable);
     HashKill(&efHNUseHashTable);
     return;
 }
