@@ -3,16 +3,16 @@
  *
  * Procedures for manipulating HierNames.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -34,7 +34,7 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "extflat/extflat.h"
 #include "extflat/EFint.h"
 
-#ifdef MAGIC_WRAPPER 
+#ifdef MAGIC_WRAPPER
 #define PrintErr TxError
 #else
 #define PrintErr printf
@@ -59,7 +59,7 @@ extern void EFHNFree();
 extern void efHNInit();
 extern void efHNRecord();
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -67,7 +67,7 @@ extern void efHNRecord();
  *
  * Determine whether a HierName is of the format of a global name,
  * i.e, it ends in a '!'.
- * 
+ *
  * The Tcl version of magic further refines this to include names
  * which are defined in the global Tcl variable space.  (7.3.94):
  * also check if the array variable "globals" contains the name as
@@ -98,7 +98,48 @@ EFHNIsGlob(hierName)
     return hierName->hn_name[strlen(hierName->hn_name) - 1] == '!';
 }
 
-
+/*
+ * ----------------------------------------------------------------------------
+ *
+ * EFHNIsGND --
+ *
+ * Determine whether a HierName is the same as the global signal GND.
+ *
+ * The Tcl version of magic expands this to include names which are
+ * equal to the global Tcl variable $GND, if it is set.
+ *
+ * This is only used in substrate backwards-compatibility mode, when the
+ * substrate is not specified in the technology file.
+ *
+ * Results:
+ *	TRUE if the name is GND, false if not.
+ *
+ * Side effects:
+ *	None.
+ *
+ * ----------------------------------------------------------------------------
+ */
+
+bool
+EFHNIsGND(hierName)
+    HierName *hierName;
+{
+#ifdef MAGIC_WRAPPER
+    char *retstr;
+#endif
+
+    if (hierName->hn_parent != (HierName *)NULL) return FALSE;
+
+#ifdef MAGIC_WRAPPER
+    retstr = (char *)Tcl_GetVar(magicinterp, "GND", TCL_GLOBAL_ONLY);
+    if (retstr != NULL)
+	if (!strcmp(hierName->hn_name, retstr)) return TRUE;
+#endif
+
+    return (strcmp(hierName->hn_name, "GND!") == 0);
+}
+
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -143,7 +184,7 @@ EFHNConcat(prefix, suffix)
 
     return firstNew;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -206,7 +247,7 @@ EFStrToHN(prefix, suffixStr)
 
     return hierName;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -277,7 +318,7 @@ efHNToStrFunc(hierName, dstp)
 
     return --dstp;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -337,7 +378,7 @@ EFHNLook(prefix, suffixStr, errorStr)
 
     return he;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -389,7 +430,7 @@ EFHNConcatLook(prefix, suffix, errorStr)
     hn->hn_parent = (HierName *) NULL;
     return he;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -427,7 +468,7 @@ EFHNFree(hierName, prefix, type)
 	}
     }
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -498,7 +539,7 @@ EFHNBest(hierName1, hierName2)
 
     return (efHNLexOrder(hierName1, hierName2) > 0);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -536,7 +577,7 @@ efHNLexOrder(hierName1, hierName2)
 
     return strcmp(hierName1->hn_name, hierName2->hn_name);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -566,7 +607,7 @@ efHNFromUse(hc, prefix)
     char *srcp, *dstp;
     char name[2048], *namePtr;
     Use *u = hc->hc_use;
-    HierName *hierName;
+    HierName *hierName, *hn;
     bool hasX, hasY;
     HashEntry *he;
     unsigned size;
@@ -622,11 +663,12 @@ efHNFromUse(hc, prefix)
     }
     HashSetValue(he, (ClientData) hierName);
 
-    (void) HashFind(&efFreeHashTable, (char *) hierName);
+    for (hn = hierName; hn; hn = hn->hn_parent)
+	HashFind(&efFreeHashTable, (char *) hierName);
 
     return hierName;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -669,7 +711,7 @@ efHNUseHash(hierName)
 {
     return hierName->hn_hash + (spointertype) hierName->hn_parent;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -717,7 +759,7 @@ efHNInit(hierName, cp, endp)
 
     hierName->hn_hash = hashsum;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -777,7 +819,7 @@ efHNHash(hierName)
 
     return n;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -847,7 +889,7 @@ efHNDistKill(dist)
 
     freeMagic((char *) dist);
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -889,7 +931,7 @@ efHNBuildDistKey(prefix, dist, distKey)
     distKey->dist_min = dist->dist_min;
     distKey->dist_max = dist->dist_max;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
