@@ -4,16 +4,16 @@
  * Procedures to read a .ext file and call the procedures
  * in EFbuild.c to build up a description of each def.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -44,8 +44,8 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #ifndef MAGIC_WRAPPER
 /* This must match the definition for extDevTable in extract/ExtBasic.c */
 char *extDevTable[] = {"fet", "mosfet", "asymmetric", "bjt", "devres",
-		"devcap", "devcaprev", "diode", "pdiode", "ndiode",
-		"subckt", "rsubckt", "msubckt", "csubckt", NULL};
+		"devcap", "devcaprev", "vsource", "diode", "pdiode",
+		"ndiode", "subckt", "rsubckt", "msubckt", "csubckt", NULL};
 #endif
 
 /*
@@ -105,7 +105,7 @@ static bool efReadDef();
 /* atoCap - convert a string to a EFCapValue */
 #define	atoCap(s)	((EFCapValue)atof(s))
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -177,7 +177,8 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
     int argc, ac, n;
     CellDef *dbdef;
     EFCapValue cap;
-    char line[1024], *argv[128], *name, *attrs;
+    char *line = NULL, *argv[128], *name, *attrs;
+    int size = 0;
     int rscale = 1;	/* Multiply resistances by this */
     int cscale = 1;	/* Multiply capacitances by this */
     float lscale = 1.0;	/* Multiply lambda by this */
@@ -207,7 +208,7 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
 	    if (sptr) {
 		*sptr = '\0';
 		inf = PaOpen(name, "r", ".ext", filepath, EFLibPath, &efReadFileName);
-	    }	
+	    }
 	    freeMagic(filepath);
 	}
     }
@@ -237,7 +238,7 @@ efReadDef(def, dosubckt, resist, noscale, toplevel)
 
 readfile:
     efReadLineNum = 0;
-    while ((argc = efReadLine(line, sizeof line, inf, argv)) >= 0)
+    while ((argc = efReadLine(&line, &size, inf, argv)) >= 0)
     {
 	n = LookupStruct(argv[0], (LookupTable *) keyTable, sizeof keyTable[0]);
 	if (n < 0)
@@ -399,9 +400,8 @@ readfile:
 		}
 		*/
 
-		/* cap = (argc > 3) ? atoCap(argv[3]) * cscale : 0; */
-		/* 3/1/2017:  Cap adjustments now handled under SUBCAP. */
-		efBuildConnect(def, argv[1], argv[2], (double)0.0, &argv[4], argc - 4);
+		cap = (argc > 3) ? atoCap(argv[3]) * cscale : 0;
+		efBuildConnect(def, argv[1], argv[2], (double)cap, &argv[4], argc - 4);
 		break;
 
 	    /* node name R C x y layer a1 p1 a2 p2 ... [ attrs ] */
@@ -555,7 +555,7 @@ resistChanged:
 		}
 #endif
 		else
-		    EFStyle = StrDup(&EFStyle, argv[1]);
+		    StrDup(&EFStyle, argv[1]);
 		break;
 
 	    /* version version-number */
@@ -614,6 +614,7 @@ resistChanged:
 	if (inf != NULL)
 	    goto readfile;
     }
+    if (line != NULL) freeMagic(line);
 
     /* If we are considering standard cells, only the first level of	*/
     /* subcircuits is meaningful.					*/
@@ -622,7 +623,7 @@ resistChanged:
 	DoSubCircuit = FALSE;
 
     /* Read in each def that has not yet been read in */
-    
+
     HashStartSearch(&hs);
     while (he = HashNext(&def->def_uses, &hs))
     {
@@ -635,7 +636,7 @@ resistChanged:
 
     return rc;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -660,37 +661,58 @@ resistChanged:
  */
 
 int
-efReadLine(line, size, file, argv)
-    char *line;			/* Character array into which line is read */
-    int size;			/* Size of character array */
-    FILE *file;	/* Open .ext file */
-    char *argv[];		/* Vector of tokens built by efReadLine() */
+efReadLine(lineptr, sizeptr, file, argv)
+    char **lineptr;	/* Pointer to character array into which line is read */
+    int *sizeptr;	/* Pointer to size of character array */
+    FILE *file;		/* Open .ext file */
+    char *argv[];	/* Vector of tokens built by efReadLine() */
 {
     char *get, *put;
     bool inquote;
     int argc = 0;
 
+    if (*sizeptr == 0)
+    {
+	*sizeptr = 1024;
+	*lineptr = (char *)mallocMagic(*sizeptr);
+    }
+    int size = *sizeptr;
+
     /* Read one line into the buffer, joining lines when they end in '\' */
 start:
-     get = line;
-     while (size > 0)
+     get = *lineptr;
+     while (TRUE)
      {
-	efReadLineNum += 1;
+	efReadLineNum++;
 	if (fgets(get, size, file) == NULL) return (-1);
-	for (put = get; *put != '\n'; put++) size -= 1;
-	if ((put != get) && (*(put-1) == '\\'))
+	for (put = get; *put != '\n' && *put != '\0'; put++) size--;
+	if ((put != get) && (*(put - 1) == '\\'))
 	{
-	    get = put-1;
+	    get = put - 1;
 	    continue;
 	}
-	*put= '\0';
-	break;
+	*put = '\0';
+
+	if (size <= 1)
+	{
+	    char *newline;
+	    int i;
+
+	    *sizeptr += 1024;	/* Increase buffer size in 1024 byte increments */
+	    newline = (char *)mallocMagic(*sizeptr);
+	    strcpy(newline, *lineptr);
+	    put = (put - (*lineptr)) + newline;
+	    get = put;
+	    freeMagic(*lineptr);
+	    *lineptr = newline;
+	    size = 1024;
+	    efReadLineNum--;
+	}
+	else break;
     }
-    if (size == 0) efReadError("long line truncated\n");
+    get = put = *lineptr;
 
-    get = put = line;
-
-    if (*line == '#') goto start;	/* Ignore comments */
+    if (*(*lineptr) == '#') goto start;	/* Ignore comments */
 
     while (*get != '\0')
     {
@@ -728,11 +750,15 @@ start:
 		}
 	    }
 
-	    if (*get == '\\')	/* Process quoted characters literally */
-	    {
-		get++;
-		if (*get == '\0') break;
-	    }
+	    /* Process backslash characters literally unless they   */
+	    /* are followed by the end-of-line.			    */
+
+	    if (*get == '\\')
+		if (*(get + 1) == '\0')
+		{
+		    get++;
+		    break;
+		}
 
 	    /* Copy into token receiving area */
 	    *put++ = *get++;
@@ -744,7 +770,7 @@ start:
 	 */
 	if (get == argv[argc])
 	    break;
-	
+
 	/* Terminate the token and advance over the terminating character. */
 
 	if (*get != '\0') get++;	/* Careful!  could be at end of line! */
