@@ -77,6 +77,7 @@ extUniqueCell(def, option)
     CellDef *def;
     int option;
 {
+    NodeRegion *nodeList;
     LabRegion *lregList, *lastreg, processedLabel;
     LabRegion *lp;
     LabelList *ll;
@@ -87,6 +88,8 @@ extUniqueCell(def, option)
     int nwarn;
     bool isabstract;
 
+    NodeRegion *extFindNodes();	    /* Forward reference */
+
     nwarn = 0;
 
     /* Check for "LEFview", as we do not care about unique names in abstract views */
@@ -97,6 +100,11 @@ extUniqueCell(def, option)
     TxPrintf("Processing %s\n", def->cd_name);
     TxFlush();
 
+    /* Find and mark the substrate node.  This prevents "soft connections"  */
+    /* through the substrate from being separated into unique names.	    */
+
+    nodeList = extFindNodes(def, (Rect *)NULL, TRUE);
+
     /* Build up a list of nodes and assign them to tiles */
     lregList = (LabRegion *) ExtFindRegions(def, &TiPlaneRect,
 				&ExtCurStyle->exts_activeTypes,
@@ -155,6 +163,7 @@ extUniqueCell(def, option)
 
     HashKill(&labelHash);
     ExtFreeLabRegions((LabRegion *) lregList);
+    freeMagic(nodeList);
     ExtResetTiles(def, extUnInit);
     if (nwarn)
 	TxError("%s: %d warnings\n", def->cd_name, nwarn);
@@ -177,7 +186,8 @@ extMakeUnique(def, ll, lreg, lregList, labelHash, option)
     int nsuffix, nwarn;
     Label saveLab, *lab;
     Rect r;
-    int flags;
+    unsigned short flags;
+    int portno;
 
     /*
      * Make a pass through all labels for all nodes.
@@ -268,17 +278,17 @@ makeUnique:
 	    flags = ll2->ll_label->lab_flags;
 	    if (flags & PORT_DIR_MASK) {
 		int idx;
-		int portno = -1;
+		portno = -1;
 		/* Find the last port index used in the cell def */
 		for (lab = def->cd_labels; lab != NULL; lab = lab->lab_next)
 		{
-		    idx = lab->lab_flags & PORT_NUM_MASK;
+		    idx = lab->lab_port;
 		    if (idx > portno) portno = idx;
 		} 
 		portno++;
-		flags &= ~PORT_NUM_MASK;
-		flags |= portno;
 	    }
+	    else
+		portno = 0;
 
 	    lab = ll2->ll_label;
 	    saveLab = *lab;
@@ -287,7 +297,7 @@ makeUnique:
 	    (void) DBPutFontLabel(def, &saveLab.lab_rect,
 		 	saveLab.lab_font, saveLab.lab_size, saveLab.lab_rotate,
 			&saveLab.lab_offset, saveLab.lab_just, name2,
-			saveLab.lab_type, flags);
+			saveLab.lab_type, flags, (unsigned int)portno);
 	    ll2->ll_label = (Label *) NULL;
 	}
 
