@@ -695,6 +695,9 @@ defnodeVisit(node, res, cap, defdata)
     TTMaskZero(&tmask);
     TTMaskSetMask(&tmask, &DBConnectTbl[magictype]);
 
+    /* Avoid attempting to extract an implicit substrate into DEF */
+    if (node->efnode_type == TT_SPACE) return 0;
+
     DBSrConnect(def, &node->efnode_loc, &tmask, DBConnectTbl,
 		&TiPlaneRect, defNetGeometryFunc,
 		(ClientData)defdata);
@@ -946,8 +949,9 @@ defNetGeometryFunc(tile, plane, defdata)
 	if ((h != routeWidth) && (w != routeWidth))
 	{
 	    /* Diagnostic */
-	    TxPrintf("Net has width %d, default width is %d\n",
-			(h > w) ? h : w, routeWidth);
+	    TxPrintf("Net at (%d, %d) has width %d, default width is %d\n",
+			r.r_xbot, r.r_ybot,
+			(h < w) ? h : w, routeWidth);
 	}
 
 	/* Find the route orientation and centerline endpoint coordinates */
@@ -960,8 +964,13 @@ defNetGeometryFunc(tile, plane, defdata)
 	    x1 = r.r_xbot * oscale;
 	    x2 = r.r_xtop * oscale;
 	    if (routeWidth == 0) routeWidth = h;
-	    extlen = (defdata->specialmode != DO_REGULAR) ?
-			0 : (routeWidth * oscale) / 2;
+	    
+	    extlen = 0;
+	    if (defdata->specialmode == DO_REGULAR)
+	    {
+		x1 = x1 + (routeWidth / 2 * oscale);
+		x2 = x2 - (routeWidth / 2 * oscale);
+	    }
 	}
 	else	/* vertical orientation */
 	{
@@ -971,8 +980,13 @@ defNetGeometryFunc(tile, plane, defdata)
 	    y1 = r.r_ybot * oscale;
 	    y2 = r.r_ytop * oscale;
 	    if (routeWidth == 0) routeWidth = w;
-	    extlen = (defdata->specialmode != DO_REGULAR) ?
-			0 : (routeWidth * oscale) / 2;
+		
+	    extlen = 0;
+	    if (defdata->specialmode == DO_REGULAR)
+	    {
+		y1 = y1 + (routeWidth / 2 * oscale);
+		y2 = y2 - (routeWidth / 2 * oscale);
+	    }
 	}
     }
     else	/* Type is a via */
@@ -1143,7 +1157,8 @@ defNetGeometryFunc(tile, plane, defdata)
 		/* residue of the contact as the route layer	*/
 		/* type.					*/
 
-		rName = defGetType((rtype == TT_SPACE) ? r2type : rtype, NULL);
+		rName = defGetType((rtype == TT_SPACE) ? r2type : rtype, NULL,
+				LAYER_MAP_VIAS);
 
 		/* The first layer in a record may not be a via name */
 
@@ -1534,9 +1549,10 @@ defCountViaFunc(tile, cviadata)
  */
 
 char *
-defGetType(ttype, lefptr)
+defGetType(ttype, lefptr, do_vias)
     TileType ttype;
     lefLayer **lefptr;
+    bool do_vias;
 {
     HashSearch hs;
     HashEntry *he;
@@ -1552,6 +1568,10 @@ defGetType(ttype, lefptr)
 	while (he = HashNext(&LefInfo, &hs))
 	{
 	    lefl = (lefLayer *)HashGetValue(he);
+	    if (lefl && (do_vias == FALSE) && (contact == CLASS_VIA) &&
+			(lefl->info.via.lr != NULL))
+		continue;	/* Skip VIA definitions if do_vias is FALSE */
+	    
 	    if (lefl && ((contact == lefl->lefClass) ||
 			((contact == CLASS_ROUTE) && (lefl->lefClass == CLASS_MASTER))))
 		if ((lefl->type == ttype) || (lefl->obsType == ttype))
@@ -1647,7 +1667,12 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
 		    if (nAc == 0)
 		    {
 			left = (r->r_xbot + r->r_xtop - size) / 2;
-			if (left >= r->r_xbot) nAc = 1;
+			nAc = 1;
+			if (left < r->r_xbot)
+			{
+			    TxError("Warning: via size is %d but area width is %d!\n",
+					size, (r->r_xtop - r->r_xbot));
+			}
 		    }
 		    else
 			left = (r->r_xbot + r->r_xtop + sep - (nAc * pitch)) / 2;
@@ -1656,7 +1681,12 @@ defWriteVias(f, rootDef, oscale, lefMagicToLefLayer)
 		    if (nUp == 0)
 		    {
 			square.r_ybot = (r->r_ybot + r->r_ytop - size) / 2;
-			if (square.r_ybot >= r->r_ybot) nUp = 1;
+			nUp = 1;
+			if (square.r_ybot >= r->r_ybot)
+			{
+			    TxError("Warning: via size is %d but area height is %d!\n",
+					size, (r->r_ytop - r->r_ybot));
+			}
 		    }
 		    else
 			square.r_ybot = (r->r_ybot + r->r_ytop + sep - (nUp * pitch)) / 2;
@@ -1752,6 +1782,135 @@ defCountCompFunc(cellUse, total)
     return 0;	/* Keep the search going */
 }
 
+/*
+ *------------------------------------------------------------
+ *
+ * defCountPins --
+ *
+ *	First-pass function to count the number of pins
+ *	to be written to the DEF output file.
+ *
+ * Results:
+ *	The total number of pins to be written.
+ *
+ * Side Effects:
+ *	None.
+ *
+ *------------------------------------------------------------
+ */
+
+int
+defCountPins(rootDef)
+    CellDef *rootDef;
+{
+    int total;
+    Label *lab;
+
+    TxPrintf("Diagnostic:  Finding all pins of cell %s\n", rootDef->cd_name);
+
+    total = 0;
+    for (lab = rootDef->cd_labels; lab; lab = lab->lab_next)
+	if (lab->lab_flags & PORT_DIR_MASK)
+	    total++;
+
+    return total;
+}
+
+/*
+ *------------------------------------------------------------
+ *
+ * defWritePins --
+ *
+ *	Output the PINS section of the DEF file.  This
+ *	is a listing of all ports, their placement, and
+ *	name.
+ *
+ * Results:
+ *	None.
+ *
+ * Side Effects:
+ *	Output to the DEF file.
+ *
+ *------------------------------------------------------------
+ */
+
+void
+defWritePins(f, rootDef, oscale)
+    FILE *f;				/* File to write to */
+    CellDef *rootDef;			/* Cell definition to use */
+    float oscale;			/* Output scale factor */
+{
+    Label *lab;
+    int lwidth, lheight;
+    int dcenterx, dcentery;
+
+    for (lab = rootDef->cd_labels; lab; lab = lab->lab_next)
+    {
+	if (lab->lab_flags & PORT_DIR_MASK)
+	{
+	    fprintf(f, "   - %s + NET %s\n", lab->lab_text, lab->lab_text);
+	    if (lab->lab_flags & PORT_CLASS_MASK)
+	    {
+		fprintf(f, "     + DIRECTION ");
+		switch (lab->lab_flags & PORT_CLASS_MASK)
+		{
+		    case PORT_CLASS_INPUT:
+			fprintf(f, "INPUT");
+			break;
+		    case PORT_CLASS_OUTPUT:
+			fprintf(f, "OUTPUT");
+			break;
+		    case PORT_CLASS_TRISTATE:
+		    case PORT_CLASS_BIDIRECTIONAL:
+			fprintf(f, "INOUT");
+			break;
+		    case PORT_CLASS_FEEDTHROUGH:
+			fprintf(f, "FEEDTHRU");
+			break;
+		}
+		fprintf(f, "\n");
+	    }
+	    if (lab->lab_flags & PORT_USE_MASK)
+	    {
+		fprintf(f, "     + USE ");
+		switch (lab->lab_flags & PORT_USE_MASK)
+		{
+		    case PORT_USE_SIGNAL:
+			fprintf(f, "SIGNAL");
+			break;
+		    case PORT_USE_ANALOG:
+			fprintf(f, "ANALOG");
+			break;
+		    case PORT_USE_POWER:
+			fprintf(f, "POWER");
+			break;
+		    case PORT_USE_GROUND:
+			fprintf(f, "GROUND");
+			break;
+		    case PORT_USE_CLOCK:
+			fprintf(f, "CLOCK");
+			break;
+		}
+		fprintf(f, "\n");
+	    }
+
+	    lwidth = lab->lab_rect.r_xtop - lab->lab_rect.r_xbot;
+	    lheight = lab->lab_rect.r_ytop - lab->lab_rect.r_ybot;
+
+	    dcenterx = lab->lab_rect.r_xtop + lab->lab_rect.r_xbot;
+	    dcentery = lab->lab_rect.r_ytop + lab->lab_rect.r_ybot;
+
+	    fprintf(f, "     + PORT\n");
+	    fprintf(f, "        + LAYER %s ( %.10g %.10g ) ( %.10g %.10g )",
+		    DBTypeLongNameTbl[lab->lab_type],
+		    oscale * (float)(-lwidth) / 2.0, oscale * (float)(-lheight) / 2.0,
+		    oscale * (float)lwidth / 2.0, oscale * (float)lheight / 2.0);
+	    fprintf(f, "        + PLACED ( %.10g %.10g ) N ;\n",
+		    oscale * (float)dcenterx / 2.0, oscale * (float)dcentery / 2.0);
+	}
+    }
+}
+
 /*
  *------------------------------------------------------------
  *
@@ -1844,10 +2003,12 @@ defComponentFunc(cellUse, defdata)
 	nameroot = cellUse->cu_def->cd_name;
 
     fprintf(f, "   - %s %s\n      + PLACED ( %.10g %.10g ) %s ;\n",
-	cellUse->cu_id, nameroot,
-	(float)cellUse->cu_bbox.r_xbot * oscale,
-	(float)cellUse->cu_bbox.r_ybot * oscale,
-	defTransPos(&cellUse->cu_transform));
+		cellUse->cu_id, nameroot,
+		(float)(cellUse->cu_bbox.r_xbot - cellUse->cu_def->cd_bbox.r_ll.p_x)
+		* oscale,
+		(float)(cellUse->cu_bbox.r_ybot - cellUse->cu_def->cd_bbox.r_ll.p_y)
+		* oscale,
+		defTransPos(&cellUse->cu_transform));
 
     return 0;	/* Keep the search going */
 }
@@ -1870,7 +2031,8 @@ defComponentFunc(cellUse, defdata)
  */
 
 LefMapping *
-defMakeInverseLayerMap()
+defMakeInverseLayerMap(do_vias)
+    bool do_vias;
 {
     LefMapping *lefMagicToLefLayer;
     lefLayer *lefl;
@@ -1882,7 +2044,7 @@ defMakeInverseLayerMap()
     memset(lefMagicToLefLayer, 0, sizeof(LefMapping) * TT_TECHDEPBASE);
     for (i = TT_TECHDEPBASE; i < DBNumTypes; i++)
     {
-	lefname = defGetType(i, &lefl);
+	lefname = defGetType(i, &lefl, do_vias);
 	lefMagicToLefLayer[i].lefName = lefname;
 	lefMagicToLefLayer[i].lefInfo = lefl;
     }
@@ -1969,7 +2131,7 @@ DefWriteCell(def, outName, allSpecial, units)
 
     defWriteHeader(def, f, scale, units);
 
-    lefMagicToLefLayer = defMakeInverseLayerMap();
+    lefMagicToLefLayer = defMakeInverseLayerMap(LAYER_MAP_VIAS);
 
     /* Vias---magic contact areas are reported as vias. */
     total = defCountVias(def, lefMagicToLefLayer, scale);
@@ -1986,7 +2148,11 @@ DefWriteCell(def, outName, allSpecial, units)
     fprintf(f, "END COMPONENTS\n\n");
 
     /* Pins---assume no pins (for now) */
-    fprintf(f, "PINS 0 ;\nEND PINS\n\n");
+    total = defCountPins(def);
+    fprintf(f, "PINS %d ;\n", total);
+    if (total > 0)
+	defWritePins(f, def, scale);
+    fprintf(f, "END PINS\n\n");
 
     /* Count the number of nets and "special" nets */
     nets = defCountNets(def, allSpecial);
@@ -2006,8 +2172,8 @@ DefWriteCell(def, outName, allSpecial, units)
     fprintf(f, "END NETS\n\n");
 
     if (nets.has_nets) {
-	EFFlatDone();
-	EFDone();
+	EFFlatDone(NULL);
+	EFDone(NULL);
     }
 
     fprintf(f, "END DESIGN\n\n");
