@@ -1339,7 +1339,7 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 	if (lab->lab_flags & PORT_DIR_MASK)
 	{
 	    curport++;
-	    idx = lab->lab_flags & PORT_NUM_MASK;
+	    idx = (int)lab->lab_port;
 	    if (idx > maxport)
 		maxport = idx;
 	}
@@ -1356,7 +1356,7 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 	    for (lab = def->cd_labels; lab != NULL; lab = lab->lab_next)
 		if (lab->lab_flags & PORT_DIR_MASK)
 		    if (!(lab->lab_flags & PORT_VISITED))
-			if ((lab->lab_flags & PORT_NUM_MASK) == idx)
+			if (lab->lab_port == idx)
 			    break;
 	}
 	else
@@ -1432,7 +1432,7 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 		    {
 			Rect psetback;
 			GEO_EXPAND(&boundary, -pinonly, &psetback);
-			SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits);
+			SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits, NULL);
 		    }
 		}
 
@@ -1461,14 +1461,14 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 
 		SelectNet(&scx, lab->lab_type, 0, NULL, FALSE);
 		GEO_EXPAND(&boundary, -setback, &carea);
-		SelRemoveArea(&carea, &DBAllButSpaceAndDRCBits);
+		SelRemoveArea(&carea, &DBAllButSpaceAndDRCBits, NULL);
 
 		/* Apply any additional setback from the "-pinonly" option */
 		if (pinonly > setback)
 		{
 		    Rect psetback;
 		    GEO_EXPAND(&boundary, -pinonly, &psetback);
-		    SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits);
+		    SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits, NULL);
 		}
 
 		/* Paint over the label area so that labels do not simply   */
@@ -1487,7 +1487,7 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 		{
 		    Rect psetback;
 		    GEO_EXPAND(&boundary, -pinonly, &psetback);
-		    SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits);
+		    SelRemoveArea(&psetback, &DBAllButSpaceAndDRCBits, NULL);
 
 		    /* Paint over the label area so that labels do not simply   */
 		    /* disappear by being inside the setback area.		*/
@@ -1602,15 +1602,42 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 	    		lefWriteGeometry, (ClientData) &lc);
 	        lc.lefMode = LEF_MODE_PORT;
 	    }
+
+	    /* Check if any other ports are already contained in this selection.    */
+	    /* If so, mark them as visited.  Use lefFindTopmost(), which is just a  */
+	    /* routine that stops the search by returning 1 when something is found */
+
+	    for (tlab = lab->lab_next; tlab != (Label *)NULL; tlab = tlab->lab_next)
+		if (tlab->lab_flags & PORT_DIR_MASK)
+		    if (!(tlab->lab_flags & PORT_VISITED))
+			if (tlab->lab_port == idx)
+			{
+			    TileTypeBitMask lmask;
+			    TTMaskSetOnlyType(&lmask, tlab->lab_type);
+			    pNum = DBPlane(tlab->lab_type);
+			    if (DBSrPaintArea((Tile *)NULL, lc.lefYank->cd_planes[pNum],
+				    &tlab->lab_rect, &lmask,
+				    lefFindTopmost, (ClientData)NULL))
+				tlab->lab_flags |= PORT_VISITED;
+
+			    /* For the "toplayer" option, ports on lower layers will not be */
+			    /* in the yank buffer but will still be in the selection.	    */
+			    else if (toplayer)
+				if (DBSrPaintArea((Tile *)NULL, SelectDef->cd_planes[pNum],
+					    &tlab->lab_rect, &lmask,
+					    lefFindTopmost, (ClientData)NULL))
+				    tlab->lab_flags |= PORT_VISITED;
+			}
+
 	    DBCellClearDef(lc.lefYank);
 	    lab->lab_flags |= PORT_VISITED;
 
-	    /* Check if any other ports belong to this pin */
+	    /* Check if any other unvisited ports belong to this pin */
 
 	    for (; lab != NULL; lab = lab->lab_next)
 		if (lab->lab_flags & PORT_DIR_MASK)
 		    if (!(lab->lab_flags & PORT_VISITED))
-			if ((lab->lab_flags & PORT_NUM_MASK) == idx)
+			if (lab->lab_port == idx)
 			    break;
 
 	    if (lc.numWrites > 0)
@@ -1633,7 +1660,7 @@ lefWriteMacro(def, f, scale, setback, pinonly, toplayer, domaster)
 	    for (tlab = reflab->lab_next; tlab != NULL; tlab = tlab->lab_next)
 	    {
 		if (tlab->lab_flags & PORT_DIR_MASK)
-		    if ((tlab->lab_flags & PORT_NUM_MASK) == idx)
+		    if (tlab->lab_port == idx)
 			if (strcmp(reflab->lab_text, tlab->lab_text))
 			{
 			    TxError("Index %d is used for ports \"%s\" and \"%s\"\n",
