@@ -51,6 +51,10 @@ HashTable 	ResIgnoreTable;	    /* Hash table of nodes to ignore  */
 
 HashTable 	ResIncludeTable;    /* Hash table of nodes to include  */
 
+/* Table of cells that have been processed */
+
+HashTable	ResProcessedTable;
+
 /* ResSimNode is a node read in from a sim file */
 
 HashTable 	ResNodeTable;   /* Hash table of sim file nodes   */
@@ -106,7 +110,7 @@ ExtResisForDef(celldef, resisdata)
     RDev	*oldRDev;
     HashSearch	hs;
     HashEntry	*entry;
-    devPtr	*tptr,*oldtptr;
+    devPtr	*tptr, *oldtptr;
     ResSimNode  *node;
     int		result, idx;
     char	*devname;
@@ -114,6 +118,10 @@ ExtResisForDef(celldef, resisdata)
     ResRDevList = NULL;
     ResOriginalNodes = NULL;
 
+    /* Check if this cell has been processed */
+    if (HashLookOnly(&ResProcessedTable, celldef->cd_name)) return;
+    HashFind(&ResProcessedTable, celldef->cd_name);
+
     /* Get device information from the current extraction style */
     idx = 0;
     while (ExtGetDevInfo(idx++, &devname, NULL, NULL, NULL, NULL, NULL))
@@ -621,10 +629,12 @@ typedef enum {
     resisdata.mainDef = mainDef;
 
     /* Do subcircuits (if any) first */
+    HashInit(&ResProcessedTable, INITFLATSIZE, HT_STRINGKEYS);
     if (!(ResOptionsFlags & ResOpt_Blackbox))
 	DBCellEnum(mainDef, resSubcircuitFunc, (ClientData) &resisdata);
 
     ExtResisForDef(mainDef, &resisdata);
+    HashKill(&ResProcessedTable);
 
     /* turn back on undo stuff */
     UndoEnable();
@@ -663,10 +673,12 @@ typedef enum {
  */
 
 int
-resSubcircuitFunc(cellDef, rdata)
-    CellDef *cellDef;
+resSubcircuitFunc(cellUse, rdata)
+    CellUse *cellUse;
     ResisData *rdata;
 {
+    CellDef *cellDef = cellUse->cu_def;
+
     if (DBIsSubcircuit(cellDef))
     {
 	ExtResisForDef(cellDef, rdata);
@@ -1188,8 +1200,7 @@ ResCheckSimNodes(celldef, resisdata)
 	fprintf(ResFHFile, "\n* Order of arguments to SPICE subcircuit call:\n");
 	for (lab = celldef->cd_labels; lab != NULL; lab = lab->lab_next)
 	    if (lab->lab_flags & PORT_DIR_MASK)
-		fprintf(ResFHFile, "* %d %s\n", lab->lab_flags & PORT_NUM_MASK,
-			lab->lab_text);
+		fprintf(ResFHFile, "* %d %s\n", lab->lab_port, lab->lab_text);
 
 	fprintf(ResFHFile, "\n.end\n");
     }
@@ -1495,6 +1506,60 @@ ResFixDevName(line, type, device, layoutnode)
     }
 }
 
+/*
+ *-------------------------------------------------------------------------
+ *
+ * devSortFunc ---
+ *
+ *	qsort() sorting function for gates.  See description in
+ *	ResSortByGate() below.
+ *
+ * Returns:
+ *	1 or -1 depending on comparison result.  The devices are sorted
+ *	by gate first, then source or drain.
+ *
+ * Side effects:
+ *	qsort() reorders the indexed list of which dev1 and dev2 are
+ *	components.
+ *
+ *-------------------------------------------------------------------------
+ */
+
+int
+devSortFunc(rec1, rec2)
+    devPtr **rec1, **rec2;
+{
+    devPtr *dev1 = *rec1;
+    devPtr *dev2 = *rec2;
+    RDev *rd1 = dev1->thisDev;
+    RDev *rd2 = dev2->thisDev;
+
+    if (dev1->terminal == GATE)
+	return 1;
+    else if (dev2->terminal == GATE)
+	return -1;
+    else if (rd1->gate > rd2->gate)
+    	return 1;
+    else if (rd1->gate == rd2->gate)
+    {
+	if ((dev1->terminal == SOURCE &&
+		dev2->terminal == SOURCE &&
+		rd1->drain > rd2->drain)    ||
+		(dev1->terminal == SOURCE &&
+		dev2->terminal == DRAIN &&
+		rd1->drain > rd2->source)    ||
+		(dev1->terminal == DRAIN &&
+		dev2->terminal == SOURCE &&
+		rd1->source > rd2->drain)    ||
+		(dev1->terminal == DRAIN &&
+		dev2->terminal == DRAIN &&
+		rd1->source >  rd2->source))
+	{
+	    return 1;
+	}
+    }
+    return -1;
+}
 
 /*
  *-------------------------------------------------------------------------
@@ -1515,101 +1580,29 @@ void
 ResSortByGate(DevpointerList)
     devPtr	**DevpointerList;
 {
-    int		changed = TRUE;
-    int		localchange = TRUE;
-    devPtr	*working, *current;
-    devPtr	*last = NULL, *gatelist = NULL;
+    devPtr	*working, **Devindexed;
+    int		listlen, listidx;
 
-    working = *DevpointerList;
-    while (working != NULL)
-    {
-	if (working->terminal == GATE)
-	{
-	    current = working;
-	    working = working->nextDev;
-       	    if (last == NULL)
-	    {
-		*DevpointerList = working;
-	    }
-	    else
-	    {
-	      	last->nextDev = working;
-	    }
-	    current->nextDev = gatelist;
-	    gatelist = current;
-	}
-	else
-	{
-	    last = working;
-	    working = working->nextDev;
-	}
-    }
-    while (changed == TRUE)
-    {
-	changed = localchange = FALSE;
-	working = *DevpointerList;
-	last = NULL;
-	while (working != NULL && (current = working->nextDev) != NULL)
-	{
-	    RDev *w = working->thisDev;
-	    RDev *c = current->thisDev;
+    /* Linked lists are very slow to sort.  Create an indexed list  */
+    /* and run qsort() to sort, then regenerate the links.	    */
 
-	    if (w->gate > c->gate)
-	    {
-	    	changed = TRUE;
-		localchange = TRUE;
-	    }
-	    else if (w->gate == c->gate &&
-			(working->terminal == SOURCE &&
-			current->terminal == SOURCE &&
-			w->drain > c->drain    ||
-			working->terminal == SOURCE &&
-			current->terminal == DRAIN &&
-			w->drain > c->source    ||
-			working->terminal == DRAIN &&
-			current->terminal == SOURCE &&
-			w->source > c->drain    ||
-			working->terminal == DRAIN &&
-			current->terminal == DRAIN &&
-			w->source >  c->source))
-	    {
-		changed = TRUE;
-		localchange = TRUE;
-	    }
-	    else
-	    {
-		last = working;
-		working = working->nextDev;
-		continue;
-	    }
-	    if (localchange)
-	    {
-		localchange = FALSE;
-		if (last == NULL)
-		{
-		     *DevpointerList = current;
-		}
-		else
-		{
-		     last->nextDev = current;
-		}
-		working->nextDev = current->nextDev;
-		current->nextDev = working;
-		last = current;
-	    }
-	}
-    }
-    if (working == NULL)
-    {
-	*DevpointerList = gatelist;
-    }
-    else
-    {
-     	if (working->nextDev != NULL)
-	    TxError("Bad Device pointer in sort\n");
-	else
-	    working->nextDev = gatelist;
-    }
+    listlen = 0;
+    for (working = *DevpointerList; working; working = working->nextDev) listlen++;
+    if (listlen == 0) return;
+
+    Devindexed = (devPtr **)mallocMagic(listlen * sizeof(devPtr *));
+    listidx = 0;
+    for (working = *DevpointerList; working; working = working->nextDev)
+	Devindexed[listidx++] = working;
+
+    qsort(Devindexed, (size_t)listlen, (size_t)sizeof(devPtr *), devSortFunc);
+
+    for (listidx = 0; listidx < listlen - 1; listidx++)
+	Devindexed[listidx]->nextDev = Devindexed[listidx + 1];
+    Devindexed[listidx]->nextDev = NULL;
+
+    *DevpointerList = Devindexed[0];
+    freeMagic(Devindexed);
 }
 
 /*
