@@ -6,16 +6,16 @@
  * also in the edit cell.  The procedures here are used as basic
  * building blocks for the selection commands like copy or delete.
  *
- *     ********************************************************************* 
- *     * Copyright (C) 1985, 1990 Regents of the University of California. * 
- *     * Permission to use, copy, modify, and distribute this              * 
- *     * software and its documentation for any purpose and without        * 
- *     * fee is hereby granted, provided that the above copyright          * 
- *     * notice appear in all copies.  The University of California        * 
- *     * makes no representations about the suitability of this            * 
- *     * software for any purpose.  It is provided "as is" without         * 
- *     * express or implied warranty.  Export of this software outside     * 
- *     * of the United States of America may require an export license.    * 
+ *     *********************************************************************
+ *     * Copyright (C) 1985, 1990 Regents of the University of California. *
+ *     * Permission to use, copy, modify, and distribute this              *
+ *     * software and its documentation for any purpose and without        *
+ *     * fee is hereby granted, provided that the above copyright          *
+ *     * notice appear in all copies.  The University of California        *
+ *     * makes no representations about the suitability of this            *
+ *     * software for any purpose.  It is provided "as is" without         *
+ *     * express or implied warranty.  Export of this software outside     *
+ *     * of the United States of America may require an export license.    *
  *     *********************************************************************
  */
 
@@ -28,9 +28,9 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 
 #include "utils/magic.h"
 #include "utils/geometry.h"
+#include "database/database.h"
 #include "tiles/tile.h"
 #include "utils/hash.h"
-#include "database/database.h"
 #include "windows/windows.h"
 #include "utils/main.h"
 #include "select/select.h"
@@ -38,15 +38,6 @@ static char rcsid[] __attribute__ ((unused)) = "$Header: /usr/cvsroot/magic-8.0/
 #include "utils/malloc.h"
 #include "textio/textio.h"
 
-/* Linked rectangle structure including tiletype information */
-
-typedef struct extRectList
-{
-    TileType r_type;
-    Rect r_r;
-    struct extRectList *r_next;
-} ExtRectList;
-
 /* Structure passed from top-level enumeration procedures to lower-level
  * ones:
  */
@@ -89,7 +80,7 @@ struct searg
 				 * selection with a possibly limited number
 				 * of named types.
 				 */
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -158,11 +149,14 @@ SelEnumPaint(layers, editOnly, foundNonEdit, func, clientData)
     arg.sea_rectList = NULL;
     if (foundNonEdit != NULL) *foundNonEdit = FALSE;
 
+    /* EditCellUse is referenced in selEnumPFunc1() and must be non-NULL */
+    if (EditCellUse == NULL) return 1;
+
     /* First, find all the paint in the selection that has the right
      * layers.  Use the same procedure as "dbCellUniqueTileSrFunc()"
      * so that contacts are not double-counted.
      */
-    
+
     for (plane = PL_SELECTBASE; plane < DBNumPlanes; plane++)
     {
 	arg.sea_plane = plane;
@@ -202,7 +196,7 @@ selEnumPFunc1(tile, arg)
 	else
 	    loctype = SplitLeftType(tile);
     }
-    else 
+    else
 	loctype = TiGetType(tile);
 
     if (IsSplit(tile))
@@ -213,7 +207,7 @@ selEnumPFunc1(tile, arg)
     /* If the paint doesn't have to be in the edit cell, life's pretty
      * simple:  just call the client and quit.
      */
-    
+
     if (!(arg->sea_flags & SEA_EDIT_ONLY))
     {
 	if ((*arg->sea_func)(&arg->sea_rect, arg->sea_type, arg->sea_cdarg) != 0)
@@ -275,7 +269,7 @@ selEnumPFunc1(tile, arg)
 
 int
 selEnumPFunc2(tile, arg)
-    Tile *tile;			/* Tile found in the edit cell */ 
+    Tile *tile;			/* Tile found in the edit cell */
     struct searg *arg;		/* Describes our search */
 {
     ExtRectList *lr;
@@ -377,7 +371,7 @@ selEnumPFunc2(tile, arg)
 	if (!IsSplit(tile))
 	{
 	    /* Things get messy here.  Clip against the select triangle	*/
-	    /* and decompose the result into rectangles and triangles.	*/ 
+	    /* and decompose the result into rectangles and triangles.	*/
 
 	    GrClipTriangle(&arg->sea_rect, &lr->r_r, TRUE, arg->sea_type, points, &np);
 
@@ -497,7 +491,7 @@ splitdone:
     return 0;
 }
 
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -602,7 +596,7 @@ selEnumCFunc1(scx, arg)
      * handle it specially:  just look for any use that's a top-level
      * use, then call the client for it.
      */
-    
+
     if (scx->scx_use->cu_def == SelectRootDef)
     {
 	CellUse *parent;
@@ -614,7 +608,7 @@ selEnumCFunc1(scx, arg)
 	    if (arg->sea_nonEdit != NULL) *(arg->sea_nonEdit) = TRUE;
 	    return 2;
 	}
-	
+
 	/* Find a top-level use (one with no parent). */
 
 	for (parent = SelectRootDef->cd_parents;
@@ -646,7 +640,7 @@ selEnumCFunc1(scx, arg)
      * we can't use DBTreeSrCells for this, because we don't want to
      * look at expanded/unexpanded information.
      */
-    
+
     scx2.scx_use = &dummy;
     dummy.cu_def = SelectRootDef;
     dummy.cu_id = NULL;
@@ -720,7 +714,7 @@ selEnumCFunc2(scx, arg)
 	    || (use->cu_array.ar_xsep != selUse->cu_array.ar_xsep)
 	    || (use->cu_array.ar_ysep != selUse->cu_array.ar_ysep))
 	goto checkChildren;
-    
+
     arg->sea_foundUse = use;
     arg->sea_foundTrans = scx->scx_trans;
     return 1;
@@ -732,7 +726,7 @@ selEnumCFunc2(scx, arg)
 	return 1;
     else return 0;
 }
-
+
 /*
  * ----------------------------------------------------------------------------
  *
@@ -803,7 +797,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
 	if (!TTMaskHasType(layers, selLabel->lab_type)) continue;
 
 	/* Find the label corresponding to this one in the design. */
-	
+
 	scx.scx_use = &dummy;
 	dummy.cu_def = SelectRootDef;
 	dummy.cu_id = NULL;
@@ -831,7 +825,7 @@ SelEnumLabels(layers, editOnly, foundNonEdit, func, clientData)
 	/* If only edit-cell labels are wanted, check this label's
 	 * parentage.
 	 */
-	
+
 	if (editOnly && (arg.sea_foundUse->cu_def != EditCellUse->cu_def))
 	{
 	    if (foundNonEdit != NULL) *foundNonEdit = TRUE;
